#include "crefl.1.7.1_mersi.h"
#include <math.h>
#include <getopt.h>

enum { OPT_BANDS = 1, OPT_RANGE, OPT_OUTFILE, OPT_MAXSOLZ };

static char indexlocator[Nitems] = { 0, 1, 0, 1, 2, 3, 4, 0, 1, 2, 3, 4, 5, 7,
    9, 10, 0, 0, 0, 0, 0, 0
};

static char numtypelocator[Nitems] = { DFNT_UINT16, DFNT_UINT16, DFNT_UINT16,
    DFNT_UINT16, DFNT_UINT16, DFNT_UINT16, DFNT_UINT16,
    DFNT_UINT16, DFNT_UINT16, DFNT_UINT16, DFNT_UINT16,
    DFNT_UINT16, DFNT_UINT16, DFNT_UINT16, DFNT_UINT16,
    DFNT_UINT16, DFNT_INT16, DFNT_INT16, DFNT_INT16, DFNT_INT16,
    DFNT_FLOAT32, DFNT_FLOAT32
};

static char *SDSlocatorQKM[Nitems] = { "EV_250_RefSB", "EV_250_RefSB",
    "EV_500_RefSB", "EV_500_RefSB", "EV_500_RefSB",
    "EV_500_RefSB", "EV_500_RefSB", "EV_1KM_RefSB", "EV_1KM_RefSB",
    "EV_1KM_RefSB", "EV_1KM_RefSB", "EV_1KM_RefSB", "EV_1KM_RefSB",
    "EV_1KM_RefSB", "EV_1KM_RefSB", "EV_1KM_RefSB", "SolarZenith",
    "SensorZenith", "SolarAzimuth", "SensorAzimuth", "Longitude",
    "Latitude"
};

static char *SDSlocatorHKM[Nitems] = { "EV_250_Aggr500_RefSB",
    "EV_250_Aggr500_RefSB", "EV_500_RefSB", "EV_500_RefSB",
    "EV_500_RefSB", "EV_500_RefSB", "EV_500_RefSB",
    "EV_1KM_RefSB", "EV_1KM_RefSB", "EV_1KM_RefSB",
    "EV_1KM_RefSB", "EV_1KM_RefSB", "EV_1KM_RefSB", "EV_1KM_RefSB",
    "EV_1KM_RefSB", "EV_1KM_RefSB", "SolarZenith",
    "SensorZenith", "SolarAzimuth", "SensorAzimuth", "Longitude",
    "Latitude"
};

static char *SDSlocator1KM[Nitems] = { "EV_250_Aggr1km_RefSB",
    "EV_250_Aggr1km_RefSB", "EV_500_Aggr1km_RefSB",
    "EV_500_Aggr1km_RefSB", "EV_500_Aggr1km_RefSB",
    "EV_500_Aggr1km_RefSB", "EV_500_Aggr1km_RefSB",
    "EV_1KM_RefSB", "EV_1KM_RefSB", "EV_1KM_RefSB", "EV_1KM_RefSB",
    "EV_1KM_RefSB", "EV_1KM_RefSB",
    "EV_1KM_RefSB", "EV_1KM_RefSB", "EV_1KM_RefSB", "SolarZenith",
    "SensorZenith", "SolarAzimuth", "SensorAzimuth", "Longitude",
    "Latitude"
};

void mersi_process_options (int main_argc, char *main_argv [], crefl_in_options_st *options_val_st, char **filename, unsigned char *process)
{
    int c, j, ftype;
    int option_index = 0;
    
    struct option long_options [] = {
		{"1km", no_argument, &(options_val_st->output1km), 1},
		{"500m", no_argument, &(options_val_st->output500m), 1},
		{"append", no_argument, &(options_val_st->append), 1},
		{"bands", required_argument, (int *) NULL, OPT_BANDS},
		{"gzip", no_argument, &(options_val_st->gzip), 1},
		{"maxsolz", required_argument, (int *) NULL, OPT_MAXSOLZ},
		{"nearest", no_argument, &(options_val_st->nearest), 1},
		{"of", required_argument, (int *) NULL, OPT_OUTFILE},
		{"overwrite", no_argument, &(options_val_st->overwrite), 1},
		{"range", required_argument, (int *) NULL, OPT_RANGE},
		{"sealevel", no_argument, &(options_val_st->sealevel), 1},
		{"toa", no_argument, &(options_val_st->TOA), 1},
		{"verbose", no_argument, &(options_val_st->verbose), 1},
		{(char *) NULL, 0, (int *) NULL, 0}
	};
    
	/* Subtract main_argc with one, and start from the second element given that the first element is 
	 * the sensor parameter value */
	while ((c = getopt_long (main_argc, main_argv, "", long_options, &option_index)) >= 0) {

		switch (c) {
		case 0:
			/* do nothing for options which will have a
			 * flag set automatically by getopt_long() */
			break;

		case OPT_BANDS:
			if (mersi_parse_bands(optarg, process)) {
				fputs("Invalid band(s) specified.\n", stderr);
				exit(1);
			}
			break;

		case OPT_RANGE:
			if (sscanf(optarg, "%g,%g", &(options_val_st->reflmin), &(options_val_st->reflmax)) != 2) {
				fputs("Error parsing reflectance range.\n",
				    stderr);
				exit(1);
			}

			if (range_check(options_val_st->reflmin, 0.0F, 1.0F) ||
			    range_check(options_val_st->reflmax, 0.0F, 1.0F)
			    || (options_val_st->reflmin >= options_val_st->reflmax)) {
				fputs("Invalid reflectance range.\n", stderr);
				exit(1);
			}

			printf
			    ("Output reflectance range [%.3f,%.3f] requested.\n",
			    options_val_st->reflmin, options_val_st->reflmax);
			break;

		case OPT_MAXSOLZ:
			options_val_st->maxsolz = (float) atof(optarg);
			if (range_check(options_val_st->maxsolz, 0.0F, 90.0F)) {
				fputs("Invalid max. solar zenith angle.\n",
				    stderr);
				exit(1);
			}
			break;

		case OPT_OUTFILE:
			*filename = optarg;
			break;

		default:
			mersi_usage();
			exit(1);
		}
	}
    
	/* at least one input file must follow */
	if (optind >= main_argc) {
		mersi_usage();
		exit(1);
	}

	/* check for conflicting options */
	if (options_val_st->overwrite && options_val_st->append) {
		fputs
		    ("Options --overwrite and --append are mutually exclusive.\n",
		    stderr);
		exit(1);
	}
	if (options_val_st->sealevel && options_val_st->TOA) {
		fputs("Options --sealevel and --toa are mutually exclusive.\n",
		    stderr);
		exit(1);
	}
#ifdef DEBUG
	printf("append = %d\n", options_val_st->append);
	if (*filename)
		printf("output filename = %s\n", *filename);
	printf("output1km = %d\n", (int) options_val_st->output1km);
	printf("output500m = %d\n", (int) options_val_st->output500m);
	printf("gzip = %d\n", options_val_st->gzip);
	printf("nearest = %d\n", options_val_st->nearest);
	printf("sealevel = %d\n", options_val_st->sealevel);
	printf("TOA = %d\n", options_val_st->TOA);
	printf("Max. solar zenith angle: %g degrees\n", options_val_st->maxsolz);
	if (*filename)
		printf("Output file: %s.", *filename);
#endif

	if (options_val_st->verbose)
		puts("Verbose mode requested.");
	if (options_val_st->overwrite)
		puts("Overwriting existing output file.");
	if (options_val_st->gzip)
		puts("Gzip compression requested.");
	if (options_val_st->sealevel)
		puts("Sea-level atmospheric correction requested. Terrain height ignored.");
	if (options_val_st->TOA)
		puts("Top-of-the-atmosphere reflectance requested. No atmospheric correction.");
	if (options_val_st->output1km)
		puts("1km-resolution output requested.");
	if (options_val_st->nearest)
		puts("Interpolation disabled.");

	/* parse input file names */
	/* An offset of -1 was used for getopt_long, therefore optind must be offsetted with +1 */
	for (j = optind; j < main_argc; j++) {
		ftype = mersi_input_file_type(main_argv[j]);

		switch (ftype) {
		case MERSI_1KM:
            options_val_st->input_files [MERSI_1KM] = main_argv [j];
			break;

		case MERSI_HKM:
            options_val_st->input_files [MERSI_HKM] = main_argv [j];
			break;

		case MERSI_QKM:
            options_val_st->input_files [MERSI_QKM] = main_argv [j];
			break;

		default:
			fprintf(stderr, "Unrecognized input file \"%s\".\n",
			    main_argv[j]);
			exit(1);
			break;
		}
	}

	if (options_val_st->verbose && options_val_st->input_files [MERSI_1KM])
		printf("Input geolocation file: %s\n", options_val_st->input_files [MERSI_1KM]);

	/* output file name is mandatory */
	if (!(*filename)) {
		fputs("Missing output file name.\n", stderr);
		exit(1);
	}
#ifdef DEBUG
	if (options_val_st->input_files [MERSI_1KM])
		printf("MOD/MYD021KMfile = %s\n", options_val_st->input_files [MERSI_1KM]);
	if (options_val_st->input_files [MERSI_HKM])
		printf("MOD/MYD02HKMfile = %s\n", options_val_st->input_files [MERSI_HKM]);
	if (options_val_st->input_files [MERSI_QKM])
		printf("MOD/MYD02QKMfile = %s\n", options_val_st->input_files [MERSI_QKM]);
#endif

	/*
	 * 1KM file is mandatory for angles.
	 * HKM file is mandatory unless 1-km output is requested.
	 * QKM file is mandatory unless 500-m or 1-km output is requested.
	 */
	if (!options_val_st->input_files [MERSI_1KM] ||
	    (!options_val_st->input_files [MERSI_HKM] && !options_val_st->output1km) ||
	    (!options_val_st->input_files [MERSI_QKM] && !options_val_st->output500m && !options_val_st->output1km)) {
		fputs("Invalid combination of input files.\n", stderr);
		exit(1);
	}
}

void mersi_init_data (crefl_in_options_st *options_val_st, SDS *sds_st_arr, unsigned char *process)
{
    int nbands, ib;
    char dummy[H4_MAX_NC_NAME];
    char *attr_name;
    int32 attr_index, count, num_type;
    float64 scale_factor[Nitems], add_offset[Nitems];
    int32 MOD02QKMfile_id, MOD02HKMfile_id, MOD021KMfile_id;
    
    /* count number of bands to process */
	for (ib = nbands = 0; ib < Nbands; ib++) {
		if (process[ib] == TRUE) {
			nbands++;
        }
    }
    
    /* if no bands where specified, set manually what bands must be processed */
	if (nbands < 1) {
		process[BAND1] = process[BAND3] = process[BAND4] = TRUE;
		if (options_val_st->verbose) {
			puts("No band(s) specified.  Default is bands 1, 3, and 4.");
        }
	}
    
    /* ======================================================================================================= */
	/* Open MERSI HDF input files */
    /* ======================================================================================================= */
	if (options_val_st->input_files [MERSI_QKM] && (!options_val_st->output500m) &&
	    !options_val_st->output1km &&
	    (MOD02QKMfile_id = SDstart(options_val_st->input_files [MERSI_QKM], DFACC_READ)) == -1) {
		fprintf(stderr, "Cannot open input file %s.\n", options_val_st->input_files [MERSI_QKM]);
		exit(1);
	}
    
	if (options_val_st->input_files [MERSI_HKM] && (!options_val_st->output1km) &&
	    (MOD02HKMfile_id = SDstart(options_val_st->input_files [MERSI_HKM], DFACC_READ)) == -1) {
		fprintf(stderr, "Cannot open input file %s.\n", options_val_st->input_files [MERSI_HKM]);
		exit(1);
	}
    
	if (options_val_st->input_files [MERSI_1KM] &&
	    (MOD021KMfile_id = SDstart(options_val_st->input_files [MERSI_1KM], DFACC_READ)) == -1) {
		fprintf(stderr, "Cannot open input file %s.\n", options_val_st->input_files [MERSI_1KM]);
		exit(1);
	}

    options_val_st->input_file_id_arr [MERSI_1KM] = MOD021KMfile_id;
    options_val_st->input_file_id_arr [MERSI_HKM] = MOD02HKMfile_id;
    options_val_st->input_file_id_arr [MERSI_QKM] = MOD02QKMfile_id;
    
    /* ======================================================================================================= */
    /* Make necessary preparations to open SDS arrays                                                          */
    /* The following are hard coded - the files that are associated with the band numbers are set              */
    /* ======================================================================================================= */
	if (options_val_st->output500m) {
		sds_st_arr[BAND1].file_id = sds_st_arr[BAND2].file_id = MOD02HKMfile_id;
		sds_st_arr[BAND1].filename = sds_st_arr[BAND2].filename = options_val_st->input_files [MERSI_HKM];
	} else {
		if (options_val_st->output1km) {
			sds_st_arr[BAND1].file_id = sds_st_arr[BAND2].file_id = MOD021KMfile_id;
			sds_st_arr[BAND1].filename = sds_st_arr[BAND2].filename =options_val_st->input_files [MERSI_HKM];
		} else {
			sds_st_arr[BAND1].file_id = sds_st_arr[BAND2].file_id = MOD02QKMfile_id;
			sds_st_arr[BAND1].filename = sds_st_arr[BAND2].filename = options_val_st->input_files [MERSI_QKM];
		}
	}

	if (options_val_st->output1km) {
		sds_st_arr[BAND3].file_id = sds_st_arr[BAND4].file_id =
		    sds_st_arr[BAND5].file_id = sds_st_arr[BAND6].file_id =
		    sds_st_arr[BAND7].file_id = MOD021KMfile_id;
		sds_st_arr[BAND3].filename = sds_st_arr[BAND4].filename =
		    sds_st_arr[BAND5].filename = sds_st_arr[BAND6].filename =
		    sds_st_arr[BAND7].filename = options_val_st->input_files [MERSI_1KM];
	} else {
		sds_st_arr[BAND3].file_id = sds_st_arr[BAND4].file_id =
		    sds_st_arr[BAND5].file_id = sds_st_arr[BAND6].file_id =
		    sds_st_arr[BAND7].file_id = MOD02HKMfile_id;
		sds_st_arr[BAND3].filename = sds_st_arr[BAND4].filename =
		    sds_st_arr[BAND5].filename = sds_st_arr[BAND6].filename =
		    sds_st_arr[BAND7].filename = options_val_st->input_files [MERSI_HKM];
	}

	sds_st_arr[BAND8].file_id = sds_st_arr[SOLZ].file_id = sds_st_arr[SOLA].file_id =
	    sds_st_arr[SENZ].file_id = sds_st_arr[SENA].file_id = sds_st_arr[LON].file_id =
	    sds_st_arr[LAT].file_id = MOD021KMfile_id;
	sds_st_arr[BAND8].filename = sds_st_arr[SOLZ].filename = sds_st_arr[SOLA].filename =

	sds_st_arr[SENZ].filename = sds_st_arr[SENA].filename = sds_st_arr[LON].filename =
	    sds_st_arr[LAT].filename = options_val_st->input_files [MERSI_1KM];

	sds_st_arr[BAND9].file_id = sds_st_arr[BAND10].file_id = sds_st_arr[BAND11].file_id =
	    sds_st_arr[BAND12].file_id = sds_st_arr[BAND13].file_id =
	    sds_st_arr[BAND14].file_id = sds_st_arr[BAND15].file_id =
	    sds_st_arr[BAND16].file_id = MOD021KMfile_id;
	sds_st_arr[BAND9].filename = sds_st_arr[BAND10].filename = sds_st_arr[BAND11].filename =
	    sds_st_arr[BAND12].filename = sds_st_arr[BAND13].filename =
	    sds_st_arr[BAND14].filename = sds_st_arr[BAND15].filename =
	    sds_st_arr[BAND16].filename = options_val_st->input_files [MERSI_1KM];

    /* ======================================================================================================= */
    /* Opening SDS arrays */
    /* ======================================================================================================= */
	for (ib = 0; ib < Nitems; ib++) {
		/* initializing these fields will simplify releasing memory later */
		sds_st_arr[ib].data = sds_st_arr[ib].fillvalue = (void *) NULL;
        
        /* Set the band number in the SDS structure */
        sds_st_arr[ib].crefl_band_number = ib;
        sds_st_arr[ib].sensor_band_number = ib;
        
		if (ib < Nbands && !process[ib]) {
			sds_st_arr[ib].id = -1;
			continue;
		}
        
		if (options_val_st->output500m) {
			sds_st_arr[ib].name = SDSlocatorHKM[ib];
		} else if (options_val_st->output1km) {
			sds_st_arr[ib].name = SDSlocator1KM[ib];
		} else {
			sds_st_arr[ib].name = SDSlocatorQKM[ib];
        }

		if ((sds_st_arr[ib].index = SDnametoindex(sds_st_arr[ib].file_id, sds_st_arr[ib].name)) == -1) {
			fprintf(stderr, "Cannot find SDS %s in file %s.\n",
			    sds_st_arr[ib].name, sds_st_arr[ib].filename);
			continue;
		}
		if ((sds_st_arr[ib].id =
			SDselect(sds_st_arr[ib].file_id, sds_st_arr[ib].index)) == -1) {
			fprintf(stderr, "Cannot select SDS no. %d\n",
			    sds_st_arr[ib].index);
			if (ib < Nbands)
				process[ib] = FALSE;
			continue;
		}

		/*
		 * Original code passed sds_st_arr[ib].name as destination for SDS name in call to SDgetinfo().
		 * This was causing a core dump, apparently because SDgetinfo() writes some additional
		 * characters beyond the terminating null at the end of the SDS name, so I replaced
		 * the argument with a dummy character array.
		 */
		if (SDgetinfo
		    (sds_st_arr[ib].id, dummy, &sds_st_arr[ib].rank, sds_st_arr[ib].dim_sizes,
			&sds_st_arr[ib].num_type, &sds_st_arr[ib].n_attr) == -1) {
			fprintf(stderr,
			    "Can't get info from SDS \"%s\" in file %s.\n",
			    sds_st_arr[ib].name, sds_st_arr[ib].filename);
			SDendaccess(sds_st_arr[ib].id);
			sds_st_arr[ib].id = -1;
			if (ib < Nbands)
				process[ib] = FALSE;
			continue;
		}

		sds_st_arr[ib].factor = 1;
		attr_name = "reflectance_scales";
		if ((attr_index = SDfindattr(sds_st_arr[ib].id, attr_name)) != -1 &&
		    SDattrinfo(sds_st_arr[ib].id, attr_index, dummy, &num_type,
			&count) != -1
		    && SDreadattr(sds_st_arr[ib].id, attr_index, scale_factor) != -1)
			sds_st_arr[ib].factor =
			    ((float32 *) scale_factor)[(int)indexlocator[ib]];
		else {
			attr_name = "scale_factor";
			if ((attr_index =
				SDfindattr(sds_st_arr[ib].id, attr_name)) != -1
			    && SDattrinfo(sds_st_arr[ib].id, attr_index, dummy,
				&num_type, &count) != -1
			    && SDreadattr(sds_st_arr[ib].id, attr_index,
				scale_factor) != -1)
				sds_st_arr[ib].factor = *scale_factor;
		}

		sds_st_arr[ib].offset = 0;
		attr_name = "reflectance_offsets";
		if ((attr_index = SDfindattr(sds_st_arr[ib].id, attr_name)) != -1 &&
		    SDattrinfo(sds_st_arr[ib].id, attr_index, dummy, &num_type,
			&count) != -1
		    && SDreadattr(sds_st_arr[ib].id, attr_index, add_offset) != -1)
			sds_st_arr[ib].offset =
			    ((float32 *) add_offset)[(int)indexlocator[ib]];
		else {
			attr_name = "add_offset";
			if ((attr_index =
				SDfindattr(sds_st_arr[ib].id, attr_name)) != -1
			    && SDattrinfo(sds_st_arr[ib].id, attr_index, dummy,
				&num_type, &count) != -1
			    && SDreadattr(sds_st_arr[ib].id, attr_index,
				add_offset) != -1)
				sds_st_arr[ib].offset = *add_offset;
		}

		sds_st_arr[ib].fillvalue =
		    (void *) malloc(1 * DFKNTsize(sds_st_arr[ib].num_type));
		if (SDgetfillvalue(sds_st_arr[ib].id, sds_st_arr[ib].fillvalue) != 0) {
			fprintf(stderr,
			    "Cannot read fill value of SDS \"%s\".\n",
			    sds_st_arr[ib].name);
			exit(1);
		}

		switch (sds_st_arr[ib].rank) {
		case 2:
			sds_st_arr[ib].Nl = sds_st_arr[ib].dim_sizes[0];
			sds_st_arr[ib].Np = sds_st_arr[ib].dim_sizes[1];
			sds_st_arr[ib].rowsperscan =
			    (int) (MERSI_NUM1KMROWPERSCAN * sds_st_arr[ib].Np / (float) MERSI_NUM1KMCOLPERSCAN + 0.5);
			sds_st_arr[ib].start[1] = 0;
			sds_st_arr[ib].edges[0] = sds_st_arr[ib].rowsperscan;
			sds_st_arr[ib].edges[1] = sds_st_arr[ib].Np;
			break;
		case 3:
			sds_st_arr[ib].Nl = sds_st_arr[ib].dim_sizes[1];
			sds_st_arr[ib].Np = sds_st_arr[ib].dim_sizes[2];
			sds_st_arr[ib].rowsperscan =
			    (int) (MERSI_NUM1KMROWPERSCAN * sds_st_arr[ib].Np / (float) MERSI_NUM1KMCOLPERSCAN + 0.5);
			sds_st_arr[ib].start[0] = indexlocator[ib];
			sds_st_arr[ib].start[2] = 0;
			sds_st_arr[ib].edges[0] = 1;
			sds_st_arr[ib].edges[1] = sds_st_arr[ib].rowsperscan;
			sds_st_arr[ib].edges[2] = sds_st_arr[ib].Np;
			break;
		default:
			fprintf(stderr, "SDS rank must be 2 or 3.\n");
			continue;
		}
        
		if (options_val_st->verbose)
			printf
			    ("SDS \"%s\": %dx%d   scale factor: %g  offset: %g\n",
			    sds_st_arr[ib].name, sds_st_arr[ib].Np, sds_st_arr[ib].Nl,
			    sds_st_arr[ib].factor, sds_st_arr[ib].offset);
		if (sds_st_arr[ib].num_type != numtypelocator[ib]) {
			fprintf(stderr,
			    "SDS \"%s\" has not the expected data type.\n",
			    sds_st_arr[ib].name);
			exit(-1);
		}
		sds_st_arr[ib].data =
		    malloc(sds_st_arr[ib].Np * sds_st_arr[ib].rowsperscan *
		    DFKNTsize(sds_st_arr[ib].num_type));
		if (!sds_st_arr[ib].data) {
			(void) fputs("Error allocating memory.\n", stderr);
			exit(1);
		}
	}
}

void mersi_finalize_data (crefl_in_options_st *options_val_st, SDS *sds_st_arr)
{
    int ib;
    
    for (ib = 0; ib < Nitems; ib++) {
		if (sds_st_arr[ib].id != -1) {
			SDendaccess(sds_st_arr[ib].id);
        }
    }
    
    for (ib = 0; ib < NR_FILES; ib++) {
        if (options_val_st->input_files [ib]) {
            SDend (options_val_st->input_file_id_arr [ib]);
        }
    }
    
    for (ib = 0; ib < Nitems; ib++) {
		if (sds_st_arr[ib].fillvalue) {
			free (sds_st_arr[ib].fillvalue);
        }
        
		if (sds_st_arr[ib].data) {
			free (sds_st_arr[ib].data);
        }
	}
}

void mersi_usage (void)
{
	fputs("Usage:\n", stderr);
	fputs
	    ("crefl <mersi | modis | viirs> [--verbose] [--1km|--500m] [--nearest] [--toa|--sealevel]\n"
	    "      [--gzip] [--maxsolz=angle] [--range=min,max] [--overwrite|--append]\n"
	    "      [--bands=<band1,band2,band3,...>] --of=<output file>\n"
	    "      <MOD021KM|MOD02CRS|MOD09CRS file> [<MOD02HKM file>] [<MOD02QKM file>]\n",
	    stderr);

	fprintf(stderr, "Version %s, compiled %s %s.\n",
	    PROCESS_VERSION_NUMBER, __DATE__, __TIME__);
}

/*
Parse MERSI L1B or coarse-resolution input file names to identify
expected spatial resolution of the MERSI swath present in file.
*/
int mersi_input_file_type (char *filename)
{
	char *ptr;
	int maxlen;

	if ((ptr = strrchr(filename, '/')))
		/* move to first character after last forward slash */
		ptr++;
	else
		/* no slash in file name */
		ptr = filename;

	/* all prefixes we'll recognize have this length */
	maxlen = strlen("M?D0????.");

	if (strncmp(ptr, "MOD021KM.", maxlen) == 0)
		return MERSI_1KM;
	if (strncmp(ptr, "MYD021KM.", maxlen) == 0)
		return MERSI_1KM;

	if (strncmp(ptr, "MOD02HKM.", maxlen) == 0)
		return MERSI_HKM;
	if (strncmp(ptr, "MYD02HKM.", maxlen) == 0)
		return MERSI_HKM;

	if (strncmp(ptr, "MOD02QKM.", maxlen) == 0)
		return MERSI_QKM;
	if (strncmp(ptr, "MYD02QKM.", maxlen) == 0)
		return MERSI_QKM;

	if (strncmp(ptr, "MOD02CRS.", maxlen) == 0)
		return MERSI_1KM;
	if (strncmp(ptr, "MYD02CRS.", maxlen) == 0)
		return MERSI_1KM;
	if (strncmp(ptr, "MOD09CRS.", maxlen) == 0)
		return MERSI_1KM;
	if (strncmp(ptr, "MYD09CRS.", maxlen) == 0)
		return MERSI_1KM;

	return INPUT_UNKNOWN;
}

int mersi_clause_solz_neg_l1bdata (SDS *solz_st, SDS *l1bdata_st, SDS *outsds_st, int crsidx, int idx)
{
    int16 *solz = (int16 *)solz_st->data;
    int16 *solzfill = (int16 *)solz_st->fillvalue;
    int16 *l1bdata = (int16 *)l1bdata_st->data;
    int16 *outdata = (int16 *)outsds_st->data;
    int16 *outdata_fill = (int16 *)outsds_st->fillvalue;
    
    if (solz [crsidx] == *solzfill || l1bdata [idx] < 0) {
        if (l1bdata [idx] == MERSI_MISSING)
            outdata [idx] = 32768 + MERSI_MISSING;
        else if (l1bdata [idx] == MERSI_SATURATED)
            outdata [idx] = 32768 + MERSI_SATURATED;
        else
            outdata [idx] = *outdata_fill;
        
        return 1;
    }

    return 0;
}

float mersi_calculate_refl (SDS *l1bdata_st, int idx, float mus0)
{
    float refl;
    int16 *l1bdata = l1bdata_st->data;
    
    refl = (l1bdata [idx] - l1bdata_st->offset) * l1bdata_st->factor / mus0;

    return refl;
}

int mersi_test_solz_gt_maxsolz (SDS *solz_st, int idx, float maxsolz)
{   
    int16 *solz = solz_st->data;
    
    if ((solz [idx] * solz_st->factor) >= maxsolz) {
        return 1;
    }

    return 0;
}

int mersi_test_solz_ne_fillvalue (SDS *solz_st, int idx)
{
    int16 *solz = solz_st->data;
    int16 *solzfill = solz_st->fillvalue;

    if (solz [idx] != *solzfill) {
        return 1;
    }

    return 0;
}

void mersi_set_solz_to_fillvalue (SDS *solz_st, int idx)
{
    int16 *solz = solz_st->data;
    int16 *solzfill = solz_st->fillvalue;
    
    solz [idx] = *solzfill;
}

float mersi_calculate_mus (SDS *solz_st, int idx)
{
    float mus;
    int16 *solz = solz_st->data;
    
    mus = cos (solz [idx] * solz_st->factor * DEG2RAD);
    
    return mus;
}

int mersi_test_bad_val (SDS *solz_st, int crsidx11, int crsidx12, int crsidx21, int crsidx22)
{
    int bad;
    int16 *solz = solz_st->data;
    int16 *solzfill = solz_st->fillvalue;
    
    bad = (solz [crsidx11] == *solzfill) || (solz [crsidx12] == *solzfill) || (solz [crsidx21] == *solzfill) || (solz [crsidx22] == *solzfill);
    
    return bad;
}

float mersi_calculate_phi (SDS *sola_st, SDS *sena_st, int idx)
{
    float phi;
    int16 *sola = sola_st->data;
    int16 *sena = sena_st->data;
    
    phi = sola [idx] * sola_st->factor - sena [idx] * sena_st->factor;
    
    return phi;
}

float mersi_calculate_muv (SDS *senz_st, int idx)
{
    float muv;
    int16 *senz = senz_st->data;
    
    muv = cos (senz [idx] * senz_st->factor * DEG2RAD);
    
    return muv;
}

/* Read current scan line for all bands to be processed.
Returns 0 if no errors, non-zero otherwise. */
int mersi_read_scan (int iscan, SDS sds_st_arr[Nitems])
{
	int ib;

	for (ib = 0; ib < Nitems; ib++) {
		if (sds_st_arr[ib].id == -1)
			continue;

		switch (sds_st_arr[ib].rank) {
		case 2:
			sds_st_arr[ib].start[0] = iscan * sds_st_arr[ib].rowsperscan;
			break;
		case 3:
			sds_st_arr[ib].start[1] = iscan * sds_st_arr[ib].rowsperscan;
			break;
		}

		if (SDreaddata (sds_st_arr[ib].id, sds_st_arr[ib].start, NULL, sds_st_arr[ib].edges,
			sds_st_arr[ib].data) == -1) {
			fprintf(stderr, "  Can't read scan %d of SDS \"%s\"\n", iscan, sds_st_arr[ib].name);
			return 1;
		}
	}

	return 0;
}

