#ifndef CREFL171_MODIS_H
#define CREFL171_MODIS_H 1

#include "crefl.1.7.1_common.h"

#define MODIS_NUMGEOCOLPERSCAN 271
#define MODIS_NUM1KMCOLPERSCAN 1354
#define MODIS_NUM1KMROWPERSCAN 10

#define MODIS_MISSING   -2
#define MODIS_SATURATED -3

/*
 * Parameters:
 * process - An char array (Nbands), where each element indicates if that specific band number's data will 
 *           be processed or not.
 */
void modis_process_options (int main_argc, char *main_argv [], crefl_in_options_st *options_val_st, char **filename, unsigned char *process);
void modis_init_data (crefl_in_options_st *options_val_st, SDS *sds_st_arr, unsigned char *process);
void modis_finalize_data (crefl_in_options_st *options_val_st, SDS *sds_st_arr);
void modis_usage (void);

/*
Parse MODIS L1B or coarse-resolution input file names to identify
expected spatial resolution of the MODIS swath present in file.
*/
int modis_input_file_type (char *filename);

int modis_clause_solz_neg_l1bdata (SDS *solz_st, SDS *l1bdata_st, SDS *outsds_st, int crsidx, int idx);

int modis_test_solz_gt_maxsolz (SDS *solz_st, int idx, float maxsolz);
int modis_test_solz_ne_fillvalue (SDS *solz_st, int idx);
int modis_test_bad_val (SDS *solz_st, int crsidx11, int crsidx12, int crsidx21, int crsidx22);

void modis_set_solz_to_fillvalue (SDS *solz_st, int idx);

float modis_calculate_refl (SDS *l1bdata_st, int idx, float mus0);
float modis_calculate_mus (SDS *solz_st, int idx);
float modis_calculate_phi (SDS *sola_st, SDS *sena_st, int idx);
float modis_calculate_muv (SDS *senz_st, int idx);

int modis_read_scan (int iscan, SDS sds_st_arr[Nitems]);

#endif /* CREFL171_MODIS_H */

